# From `man cowsay`:
#
# There are several provided modes which change the appearance of the cow
# depending on its particular emotional/physical state.   The  -b  option
# initiates  Borg  mode;  -d  causes  the  cow to appear dead; -g invokes
# greedy mode; -p causes a state of paranoia to come  over  the  cow;  -s
# makes  the  cow  appear thoroughly stoned; -t yields a tired cow; -w is
# somewhat the opposite of -t, and initiates wired mode; -y brings on the
# cow's youthful appearance.

DEFAULT_EYES = 'oo'
DEFAULT_TONGUE = '  '

modes = {
    'borg':{
        'eyes': '==',
        'tongue': '  ',
    },
    'dead': {
        'eyes': 'xx',
        'tongue': 'U ',
    },
    'greedy': {
        'eyes': '@@',
        'tongue': '  ',
    },
    'paranoid': {
        'eyes': '@@',
        'tongue': '  ',
    },
    'stoned': {
        'eyes': 'xx',
        'tongue': '  ',
    },
    'tired': {
        'eyes': '--',
        'tongue': '  ',
    },
    'wired': {
        'eyes': '@@',
        'tongue': '  ',
    },
    'youthful': {
        'eyes': 'oo',
        'tongue': '  ',
    },
}

def getFace(mode=None, **options):
    e = options.get('e') if options.get('e') else None
    T = options.get('T') if options.get('T') else None
    if mode: 
        return modes[mode]
    else: 
        eyes = e if e else DEFAULT_EYES
        tongue = T if T else DEFAULT_TONGUE
        return eyes, tongue
        """return {
        'eyes': e if e else DEFAULT_EYES,
        'tongue': T if T else DEFAULT_TONGUE,
    }"""



def Goat2(thoughts, eyes, eye, tongue):
  return f"""
        {thoughts}
         {thoughts}
          )__(
         '|{eyes}|'________/
          |__|         |
           {tongue}||"""""""||
             ||       ||

"""


def Goat(thoughts, eyes, eye, tongue):
  return f"""
       {thoughts}
        {thoughts}
         {thoughts}  _))
           > {eye}\\     _~
           \"""'\\\\__-' \\_
              | )  _ \\ \\
             / / \`\`   w w
            w w
"""


def Www(thoughts, eyes, eye, tongue):
  return f"""
        {thoughts}   ^__^
         {thoughts}  ({eyes})\\_______
            (__)\\       )\\/\\
             {tongue} ||--WWW |
                ||     ||
"""


def Golden_eagle(thoughts, eyes, eye, tongue):
  return f"""
    {thoughts}                                       ,:=+++++???++?+=+=
     {thoughts}                               :+?????\$MUUUUUUUUUMO+??~
      {thoughts}                         :+I??\$UUUUMUMMMMUUMUMUUMUUMM???I+:
       {thoughts}                     ,+??+ZOUUMMUMMMUUUUUMUUUMUUUMUMUUMZI+?+:
        {thoughts}                 ~I?+MMUMUUUMUUUOOUMMMMMMUUUUUMMMUUUUUUMMUM$??~
                       I?+7MMMMMUUO7?+?IUMMMMMMMMUUMUUMUUUUUUUUUUMMMUMO?I
                    ~I?+MMMUUUO????+?IOUZ7?,.......,+\$\$OUMUUUUUUMUMUUUMUU+I:
                   =??\$UMUUU7++??????II???????=.....,?OUUMMMUUMUUUUUUUMMUUU+=
                 +??UUMMM7??????+??+?????+??=,...\$MUMUUUUMUUMUUUUUUUUMMUM7II??=
               ,+?IUMMMI???III?++??+?????+~....... ......MUUMUUUUUUUUUMMU7?~
               IIUMMM+?+?IUUUUMUUM7I?????????????I?+=:......MUUMMUMUMMMMUUU+~
             :?+UMMU+?+?7UMMUUUZ7\$\$7????+++????????????=.....+UMUUUUMMMMMMUZ?
             ?+UMUM???+MMMMMU?++???????????++????????++????....OMMMUMMMMUMMUI:
            +\$MMUM?+?ZMMU:\$MM???OUUU+??+???+????????????????,...UMUMUUUUMMUMM?~
            IUUUU?I?OUUU,..UU?IMMUUMUUI???+?????????????????I,..:UUMUUUUMMUMU?+
            ?UUUMUM\$UMUU~..UUUUU\$,IUUUMM7+?????????????????+?I~..UUUUUUUMMMUU+?
            ?OUMUUUUMMUI+.?UUUU=...~UMMUU\$?????+???????????????..MUMUUUUUMUMU??
           :??IUUMMUMUMMOMUU7........OUUUMMU?I????????????????I..MUMUUUUMUUMU?+
         +IIUMUO.IUUUUUUO..............?UMUMUM7??????????????+?..UUMUUUUMUUMU?=
       ,IZMMU,.:UU7:..........,UUUMUZ....MUUMMO+???????????????..UUUUUUUUUUU?:
       IZUUU:..UUUI=....... IUUUUUMMUZ,.MMUMU$?+???????????????.MUUMUUUUMUMUI
     ,+IUUM..O=..........\$UUMMMUU?~....UMMUUI?????????????????=.UMMUUUUUMMU?+
     +?UMU~............OUMMUU~..... .UUMUMM+?????????????????=.UMMUMUMUUMOI=
     ?\$MU~...    ...:MUMU=~........,UUMMMUI+????????????????+IUMMMUUUUMUU?+
     +OMU....   ...?UMU=..:~~,.....MMMUUU+?+????????????????~MMUUUUMUMMU?+~   
     ?OMU~ .. ...?UMUUUMUMUMUMUMUUUMUUMUI???????????????+?+OUUMUUMMMMUIUUUMO,
     ??UMU~.....\$MUUUOM???UMMUUUMMMUUMM7?++????????????++OMMMUMUUMMUI??UMIU+~
     :?7UUU\$...UMMM?I~,  +?MMUUMMMMMMUU?????????????+??\$UMMMMUUUMU\$?: ,??I?:
       ?IMUMUUZMU+?,      =?UMMMMMMMMO??????????????+UMUMMUMUMUU?I~
        ?+\$MUMUMU??        ?MMMMMMMMU??+???????????IUMMMUUUMUUZ?=
          ,+???ZUO?~       +ZUMMUMUMU???++??+???IUMMUMUMMUUO??~
               ,,:~=       ,?UMUMUMU???+??+?+?7UMUMUMUMUI??:
                            ?UMUMMMM?+??++?ZUUMUMUMUZ++?,                  
                            ?UMMMMMO+???MMUMUMUMUMOII=,                       
                            ?UUUMUUZOMUUMUMMUMM+??=
                            ?UMMUMMUUUMUMM\$???~
                           ,?UMUUMUUU\$?+?~:                                  
                           :IUUUM?+?I=:                                  
                           ????~,
"""


def Minotaur(thoughts, eyes, eye, tongue):
  return f"""
        {thoughts}   ^__^
         {thoughts}  ({eyes})
            (__)
           /-||-\\
           \\|\\/|/
            o==o 
            ||||
            ()()
"""


def Elephant_in_snake(thoughts, eyes, eye, tongue):
  return f"""
   {thoughts}
    {thoughts}              ....       
           ........    .      
          .            .      
         .             .      
.........              .......
..............................

Elephant inside ASCII snake
"""


def Skeleton(thoughts, eyes, eye, tongue):
  return f"""
          {thoughts}      (__)      
           {thoughts}     /{eyes}|  
            {thoughts}   (_"_)*+++++++++*
                   //I#\\\\\\\\\\\\\\\\I\\
                   I[I|I|||||I I \`
                   I\`I'///'' I I
                   I I       I I
                   ~ ~       ~ ~
                     SSkeletonleton
"""


def Supermilker(thoughts, eyes, eye, tongue):
  return f"""
  {thoughts}   ^__^
   {thoughts}  ({eyes})\\_______        ________
      (__)\\       )\\/\\    |Super |
       {tongue} ||----W |       |Milker|
          ||    UDDDDDDDDD|______|
"""


def Telebears(thoughts, eyes, eye, tongue):
  return f"""
      {thoughts}                _
       {thoughts}              (_)   <-- TeleBEARS
        {thoughts}   ^__^       / \\
         {thoughts}  ({eyes})\\_____/_\\ \\
            (__)\\  you  ) /
             {tongue} ||----w ((
                ||     ||>> 
"""


def Luke_koala(thoughts, eyes, eye, tongue):
  return f"""
  {thoughts}
   {thoughts}          .
       ___   //
     """+'{'+f"~{eye}_{eye}~"+"""}// 
      ( Y )K/  
     ()~*~()   
     (_)-(_)   
     Luke    
     Sywalker
     koala   
"""


def Hiyoko(thoughts, eyes, eye, tongue):
  return f"""
  {thoughts}
   {thoughts}
    {thoughts}
      ,､ ,._
      ﾉ ・  ヽ
     / :::   i  
    / :::    ﾞ､
   ,i:::       \`ｰ-､
   |:::           i
   !::::..        ﾉ
    \`ー――――'" 
"""


"""
Python package for cowsay.
"""

__version__ = "0.1.1"
__author__ = 'Ovlic'

from .cows import cows
from .lib import balloon
from .lib.faces import getFace
DEFAULT_COLUMNS = 40

def doIt(text, action, **options):
    nowrap = options.get('nowrap')
    wrapWidth = DEFAULT_COLUMNS if not nowrap else (options.get('wrapWidth') if options.get('wrapWidth') else None)
    filledBalloon = balloon.say(text, wrapWidth) if action == "say" else balloon.think(text, wrapWidth)
    cow = options.get('cow') if options.get('cow') else cows.defaultCow
    eyes, tongue = getFace(e=options.get('eyes'), T=options.get('tongue'), cow=cow)
    filledCow = cow(thoughts=('\\' if action == 'say' else 'o'), eyes=eyes, tongue=tongue, eye=eyes[0],)
    return filledBalloon + filledCow


def say(text, **options):
  """
        Make the cow say something.

        Parameters
        ----------
        text: :class:`str`
            Text the cow says.
        eyes: :class:`str`
            Eyes for the cow. Defaults to ``'oo'``.
        tongue: :class:`str`
            Tongue for the cow. Defaults to ``'  '``.
        nowrap: :class:`bool`
            Doesn't wrap text. Defaults to ``False``.
        wrapWidth: :class:`int`
            Wrap text. Defaults to ``40``.
        cow: :class:`cows.Cow`
            Cow to use. Defaults to :class:`cows.defaultCow`.

        Returns
        --------
        :class:`str`
            Cow string."""
  return doIt(text, 'say', **options)


def think(text, **options):
  """
        Make the cow think something.

        Parameters
        ----------
        text: :class:`str`
            Text the cow thinks.
        eyes: :class:`str`
            Eyes for the cow.
        tongue: :class:`str`
            Tongue for the cow.
        nowrap: :class:`bool`
            Doesn't wrap text. Defaults to ``False``.
        wrapWidth: :class:`int`
            Wrap text. Defaults to ``40``.
        cow: :class:`cows.Cow`
            Cow to use. Defaults to :class:`cows.defaultCow`.

        Returns
        --------
        :class:`str`
            Cow string."""
  return doIt(text, 'think', **options)




def Roflcopter(thoughts, eyes, eye, tongue):
  return f"""
   {thoughts}
    {thoughts}
 ROFL:ROFL:ROFL:ROFL
         _^___
 L    __/   {eyes} \\    
LOL===__        \\ 
 L      \\________]
         I   I  {tongue}
        --------/
"""



from .lib.cows.C3PO import C3PO
from .lib.cows.R2_D2 import R2_D2
from .lib.cows.USA import USA
from .lib.cows.ackbar import Ackbar
from .lib.cows.aperture import Aperture
from .lib.cows.aperture_blank import Aperture_blank
from .lib.cows.armadillo import Armadillo
from .lib.cows.atat import Atat
from .lib.cows.atom import Atom
from .lib.cows.awesome_face import Awesome_face
from .lib.cows.banana import Banana
from .lib.cows.bearface import Bearface
from .lib.cows.beavis_zen import Beavis_zen
from .lib.cows.bees import Bees
from .lib.cows.bill_the_cat import Bill_the_cat
from .lib.cows.biohazard import Biohazard
from .lib.cows.bishop import Bishop
from .lib.cows.black_mesa import Black_mesa
from .lib.cows.bong import Bong
from .lib.cows.box import Box
from .lib.cows.broken_heart import Broken_heart
from .lib.cows.bud_frogs import Bud_frogs
from .lib.cows.bunny import Bunny
from .lib.cows.cake import Cake
from .lib.cows.cake_with_candles import Cake_with_candles
from .lib.cows.cat import Cat
from .lib.cows.cat2 import Cat2
from .lib.cows.catfence import Catfence
from .lib.cows.charizardvice import Charizardvice
from .lib.cows.charlie import Charlie
from .lib.cows.cheese import Cheese
from .lib.cows.chessmen import Chessmen
from .lib.cows.chito import Chito
from .lib.cows.claw_arm import Claw_arm
from .lib.cows.clippy import Clippy
from .lib.cows.companion_cube import Companion_cube
from .lib.cows.cower import Cower
from .lib.cows.cowfee import Cowfee
from .lib.cows.cthulhu_mini import Cthulhu_mini
from .lib.cows.cube import Cube
from .lib.cows.daemon import Daemon
from .lib.cows.dalek import Dalek
from .lib.cows.dalek_shooting import Dalek_shooting
from .lib.cows.default import Default
from .lib.cows.docker_whale import Docker_whale
from .lib.cows.doge import Doge
from .lib.cows.dolphin import Dolphin
from .lib.cows.dragon import Dragon
from .lib.cows.dragon_and_cow import Dragon_and_cow
from .lib.cows.ebi_furai import Ebi_furai
from .lib.cows.elephant import Elephant
from .lib.cows.elephant2 import Elephant2
from .lib.cows.elephant_in_snake import Elephant_in_snake
from .lib.cows.explosion import Explosion
from .lib.cows.eyes import Eyes
from .lib.cows.fat_banana import Fat_banana
from .lib.cows.fat_cow import Fat_cow
from .lib.cows.fence import Fence
from .lib.cows.fire import Fire
from .lib.cows.flaming_sheep import Flaming_sheep
from .lib.cows.fox import Fox
from .lib.cows.ghost import Ghost
from .lib.cows.ghostbusters import Ghostbusters
from .lib.cows.glados import Glados
from .lib.cows.goat import Goat
from .lib.cows.goat2 import Goat2
from .lib.cows.golden_eagle import Golden_eagle
from .lib.cows.hand import Hand
from .lib.cows.happy_whale import Happy_whale
from .lib.cows.hedgehog import Hedgehog
from .lib.cows.hellokitty import Hellokitty
from .lib.cows.hippie import Hippie
from .lib.cows.hiya import Hiya
from .lib.cows.hiyoko import Hiyoko
from .lib.cows.homer import Homer
from .lib.cows.hypno import Hypno
from .lib.cows.ibm import Ibm
from .lib.cows.iwashi import Iwashi
from .lib.cows.jellyfish import Jellyfish
from .lib.cows.karl_marx import Karl_marx
from .lib.cows.kilroy import Kilroy
from .lib.cows.king import King
from .lib.cows.kiss import Kiss
from .lib.cows.kitten import Kitten
from .lib.cows.kitty import Kitty
from .lib.cows.knight import Knight
from .lib.cows.koala import Koala
from .lib.cows.kosh import Kosh
from .lib.cows.lamb import Lamb
from .lib.cows.lamb2 import Lamb2
from .lib.cows.lightbulb import Lightbulb
from .lib.cows.lobster import Lobster
from .lib.cows.lollerskates import Lollerskates
from .lib.cows.luke_koala import Luke_koala
from .lib.cows.mailchimp import Mailchimp
from .lib.cows.maze_runner import Maze_runner
from .lib.cows.mech_and_cow import Mech_and_cow
from .lib.cows.meow import Meow
from .lib.cows.milk import Milk
from .lib.cows.minotaur import Minotaur
from .lib.cows.mona_lisa import Mona_lisa
from .lib.cows.moofasa import Moofasa
from .lib.cows.mooghidjirah import Mooghidjirah
from .lib.cows.moojira import Moojira
from .lib.cows.moose import Moose
from .lib.cows.mule import Mule
from .lib.cows.mutilated import Mutilated
from .lib.cows.nyan import Nyan
from .lib.cows.octopus import Octopus
from .lib.cows.okazu import Okazu
from .lib.cows.owl import Owl
from .lib.cows.pawn import Pawn
from .lib.cows.periodic_table import Periodic_table
from .lib.cows.personality_sphere import Personality_sphere
from .lib.cows.pinball_machine import Pinball_machine
from .lib.cows.psychiatrichelp import Psychiatrichelp
from .lib.cows.psychiatrichelp2 import Psychiatrichelp2
from .lib.cows.pterodactyl import Pterodactyl
from .lib.cows.queen import Queen
from .lib.cows.radio import Radio
from .lib.cows.ren import Ren
from .lib.cows.renge import Renge
from .lib.cows.robot import Robot
from .lib.cows.robotfindskitten import Robotfindskitten
from .lib.cows.roflcopter import Roflcopter
from .lib.cows.rook import Rook
from .lib.cows.sachiko import Sachiko
from .lib.cows.satanic import Satanic
from .lib.cows.seahorse import Seahorse
from .lib.cows.seahorse_big import Seahorse_big
from .lib.cows.sheep import Sheep
from .lib.cows.shikato import Shikato
from .lib.cows.shrug import Shrug
from .lib.cows.skeleton import Skeleton
from .lib.cows.small import Small
from .lib.cows.smiling_octopus import Smiling_octopus
from .lib.cows.snoopy import Snoopy
from .lib.cows.snoopyhouse import Snoopyhouse
from .lib.cows.snoopysleep import Snoopysleep
from .lib.cows.spidercow import Spidercow
from .lib.cows.squid import Squid
from .lib.cows.squirrel import Squirrel
from .lib.cows.stegosaurus import Stegosaurus
from .lib.cows.stimpy import Stimpy
from .lib.cows.sudowoodo import Sudowoodo
from .lib.cows.supermilker import Supermilker
from .lib.cows.surgery import Surgery
from .lib.cows.tableflip import Tableflip
from .lib.cows.taxi import Taxi
from .lib.cows.telebears import Telebears
from .lib.cows.template import Template
from .lib.cows.threader import Threader
from .lib.cows.threecubes import Threecubes
from .lib.cows.toaster import Toaster
from .lib.cows.tortoise import Tortoise
from .lib.cows.turkey import Turkey
from .lib.cows.turtle import Turtle
from .lib.cows.tux import Tux
from .lib.cows.tux_big import Tux_big
from .lib.cows.tweety_bird import Tweety_bird
from .lib.cows.vader import Vader
from .lib.cows.vader_koala import Vader_koala
from .lib.cows.weeping_angel import Weeping_angel
from .lib.cows.whale import Whale
from .lib.cows.wizard import Wizard
from .lib.cows.wood import Wood
from .lib.cows.world import World
from .lib.cows.www import Www
from .lib.cows.yasuna_01 import Yasuna_01
from .lib.cows.yasuna_02 import Yasuna_02
from .lib.cows.yasuna_03 import Yasuna_03
from .lib.cows.yasuna_03a import Yasuna_03a
from .lib.cows.yasuna_04 import Yasuna_04
from .lib.cows.yasuna_05 import Yasuna_05
from .lib.cows.yasuna_06 import Yasuna_06
from .lib.cows.yasuna_07 import Yasuna_07
from .lib.cows.yasuna_08 import Yasuna_08
from .lib.cows.yasuna_09 import Yasuna_09
from .lib.cows.yasuna_10 import Yasuna_10
from .lib.cows.yasuna_11 import Yasuna_11
from .lib.cows.yasuna_12 import Yasuna_12
from .lib.cows.yasuna_13 import Yasuna_13
from .lib.cows.yasuna_14 import Yasuna_14
from .lib.cows.yasuna_16 import Yasuna_16
from .lib.cows.yasuna_17 import Yasuna_17
from .lib.cows.yasuna_18 import Yasuna_18
from .lib.cows.yasuna_19 import Yasuna_19
from .lib.cows.yasuna_20 import Yasuna_20
from .lib.cows.ymd_udon import Ymd_udon
from .lib.cows.zen_noh_milk import Zen_noh_milk

class cows:
    """cows yay"""
    
    @property
    def C3PO(**kwargs): return C3PO(**kwargs)
    
    @property
    def R2_D2(**kwargs): return R2_D2(**kwargs)
    
    @property
    def USA(**kwargs): return USA(**kwargs)
    
    @property
    def ackbar(**kwargs): return Ackbar(**kwargs)
    
    @property
    def aperture(**kwargs): return Aperture(**kwargs)
    
    @property
    def aperture_blank(**kwargs): return Aperture_blank(**kwargs)
    
    @property
    def armadillo(**kwargs): return Armadillo(**kwargs)
    
    @property
    def atat(**kwargs): return Atat(**kwargs)
    
    @property
    def atom(**kwargs): return Atom(**kwargs)
    
    @property
    def awesome_face(**kwargs): return Awesome_face(**kwargs)
    
    @property
    def banana(**kwargs): return Banana(**kwargs)
    
    @property
    def bearface(**kwargs): return Bearface(**kwargs)
    
    @property
    def beavis_zen(**kwargs): return Beavis_zen(**kwargs)
    
    @property
    def bees(**kwargs): return Bees(**kwargs)
    
    @property
    def bill_the_cat(**kwargs): return Bill_the_cat(**kwargs)
    
    @property
    def biohazard(**kwargs): return Biohazard(**kwargs)
    
    @property
    def bishop(**kwargs): return Bishop(**kwargs)
    
    @property
    def black_mesa(**kwargs): return Black_mesa(**kwargs)
    
    @property
    def bong(**kwargs): return Bong(**kwargs)
    
    @property
    def box(**kwargs): return Box(**kwargs)
    
    @property
    def broken_heart(**kwargs): return Broken_heart(**kwargs)
    
    @property
    def bud_frogs(**kwargs): return Bud_frogs(**kwargs)
    
    @property
    def bunny(**kwargs): return Bunny(**kwargs)
    
    @property
    def cake(**kwargs): return Cake(**kwargs)
    
    @property
    def cake_with_candles(**kwargs): return Cake_with_candles(**kwargs)
    
    @property
    def cat(**kwargs): return Cat(**kwargs)
    
    @property
    def cat2(**kwargs): return Cat2(**kwargs)
    
    @property
    def catfence(**kwargs): return Catfence(**kwargs)
    
    @property
    def charizardvice(**kwargs): return Charizardvice(**kwargs)
    
    @property
    def charlie(**kwargs): return Charlie(**kwargs)
    
    @property
    def cheese(**kwargs): return Cheese(**kwargs)
    
    @property
    def chessmen(**kwargs): return Chessmen(**kwargs)
    
    @property
    def chito(**kwargs): return Chito(**kwargs)
    
    @property
    def claw_arm(**kwargs): return Claw_arm(**kwargs)
    
    @property
    def clippy(**kwargs): return Clippy(**kwargs)
    
    @property
    def companion_cube(**kwargs): return Companion_cube(**kwargs)
    
    @property
    def cower(**kwargs): return Cower(**kwargs)
    
    @property
    def cowfee(**kwargs): return Cowfee(**kwargs)
    
    @property
    def cthulhu_mini(**kwargs): return Cthulhu_mini(**kwargs)
    
    @property
    def cube(**kwargs): return Cube(**kwargs)
    
    @property
    def daemon(**kwargs): return Daemon(**kwargs)
    
    @property
    def dalek(**kwargs): return Dalek(**kwargs)
    
    @property
    def dalek_shooting(**kwargs): return Dalek_shooting(**kwargs)
    
    @property
    def default(**kwargs): return Default(**kwargs)
    
    @property
    def defaultCow(**kwargs): return Default(**kwargs)
    
    @property
    def docker_whale(**kwargs): return Docker_whale(**kwargs)
    
    @property
    def doge(**kwargs): return Doge(**kwargs)
    
    @property
    def dolphin(**kwargs): return Dolphin(**kwargs)
    
    @property
    def dragon(**kwargs): return Dragon(**kwargs)
    
    @property
    def dragon_and_cow(**kwargs): return Dragon_and_cow(**kwargs)
    
    @property
    def ebi_furai(**kwargs): return Ebi_furai(**kwargs)
    
    @property
    def elephant(**kwargs): return Elephant(**kwargs)
    
    @property
    def elephant2(**kwargs): return Elephant2(**kwargs)
    
    @property
    def elephant_in_snake(**kwargs): return Elephant_in_snake(**kwargs)
    
    @property
    def explosion(**kwargs): return Explosion(**kwargs)
    
    @property
    def eyes(**kwargs): return Eyes(**kwargs)
    
    @property
    def fat_banana(**kwargs): return Fat_banana(**kwargs)
    
    @property
    def fat_cow(**kwargs): return Fat_cow(**kwargs)
    
    @property
    def fence(**kwargs): return Fence(**kwargs)
    
    @property
    def fire(**kwargs): return Fire(**kwargs)
    
    @property
    def flaming_sheep(**kwargs): return Flaming_sheep(**kwargs)
    
    @property
    def fox(**kwargs): return Fox(**kwargs)
    
    @property
    def ghost(**kwargs): return Ghost(**kwargs)
    
    @property
    def ghostbusters(**kwargs): return Ghostbusters(**kwargs)
    
    @property
    def glados(**kwargs): return Glados(**kwargs)
    
    @property
    def goat(**kwargs): return Goat(**kwargs)
    
    @property
    def goat2(**kwargs): return Goat2(**kwargs)
    
    @property
    def golden_eagle(**kwargs): return Golden_eagle(**kwargs)
    
    @property
    def hand(**kwargs): return Hand(**kwargs)
    
    @property
    def happy_whale(**kwargs): return Happy_whale(**kwargs)
    
    @property
    def hedgehog(**kwargs): return Hedgehog(**kwargs)
    
    @property
    def hellokitty(**kwargs): return Hellokitty(**kwargs)
    
    @property
    def hippie(**kwargs): return Hippie(**kwargs)
    
    @property
    def hiya(**kwargs): return Hiya(**kwargs)
    
    @property
    def hiyoko(**kwargs): return Hiyoko(**kwargs)
    
    @property
    def homer(**kwargs): return Homer(**kwargs)
    
    @property
    def hypno(**kwargs): return Hypno(**kwargs)
    
    @property
    def ibm(**kwargs): return Ibm(**kwargs)
    
    @property
    def iwashi(**kwargs): return Iwashi(**kwargs)
    
    @property
    def jellyfish(**kwargs): return Jellyfish(**kwargs)
    
    @property
    def karl_marx(**kwargs): return Karl_marx(**kwargs)
    
    @property
    def kilroy(**kwargs): return Kilroy(**kwargs)
    
    @property
    def king(**kwargs): return King(**kwargs)
    
    @property
    def kiss(**kwargs): return Kiss(**kwargs)
    
    @property
    def kitten(**kwargs): return Kitten(**kwargs)
    
    @property
    def kitty(**kwargs): return Kitty(**kwargs)
    
    @property
    def knight(**kwargs): return Knight(**kwargs)
    
    @property
    def koala(**kwargs): return Koala(**kwargs)
    
    @property
    def kosh(**kwargs): return Kosh(**kwargs)
    
    @property
    def lamb(**kwargs): return Lamb(**kwargs)
    
    @property
    def lamb2(**kwargs): return Lamb2(**kwargs)
    
    @property
    def lightbulb(**kwargs): return Lightbulb(**kwargs)
    
    @property
    def lobster(**kwargs): return Lobster(**kwargs)
    
    @property
    def lollerskates(**kwargs): return Lollerskates(**kwargs)
    
    @property
    def luke_koala(**kwargs): return Luke_koala(**kwargs)
    
    @property
    def mailchimp(**kwargs): return Mailchimp(**kwargs)
    
    @property
    def maze_runner(**kwargs): return Maze_runner(**kwargs)
    
    @property
    def mech_and_cow(**kwargs): return Mech_and_cow(**kwargs)
    
    @property
    def meow(**kwargs): return Meow(**kwargs)
    
    @property
    def milk(**kwargs): return Milk(**kwargs)
    
    @property
    def minotaur(**kwargs): return Minotaur(**kwargs)
    
    @property
    def mona_lisa(**kwargs): return Mona_lisa(**kwargs)
    
    @property
    def moofasa(**kwargs): return Moofasa(**kwargs)
    
    @property
    def mooghidjirah(**kwargs): return Mooghidjirah(**kwargs)
    
    @property
    def moojira(**kwargs): return Moojira(**kwargs)
    
    @property
    def moose(**kwargs): return Moose(**kwargs)
    
    @property
    def mule(**kwargs): return Mule(**kwargs)
    
    @property
    def mutilated(**kwargs): return Mutilated(**kwargs)
    
    @property
    def nyan(**kwargs): return Nyan(**kwargs)
    
    @property
    def octopus(**kwargs): return Octopus(**kwargs)
    
    @property
    def okazu(**kwargs): return Okazu(**kwargs)
    
    @property
    def owl(**kwargs): return Owl(**kwargs)
    
    @property
    def pawn(**kwargs): return Pawn(**kwargs)
    
    @property
    def periodic_table(**kwargs): return Periodic_table(**kwargs)
    
    @property
    def personality_sphere(**kwargs): return Personality_sphere(**kwargs)
    
    @property
    def pinball_machine(**kwargs): return Pinball_machine(**kwargs)
    
    @property
    def psychiatrichelp(**kwargs): return Psychiatrichelp(**kwargs)
    
    @property
    def psychiatrichelp2(**kwargs): return Psychiatrichelp2(**kwargs)
    
    @property
    def pterodactyl(**kwargs): return Pterodactyl(**kwargs)
    
    @property
    def queen(**kwargs): return Queen(**kwargs)
    
    @property
    def radio(**kwargs): return Radio(**kwargs)
    
    @property
    def ren(**kwargs): return Ren(**kwargs)
    
    @property
    def renge(**kwargs): return Renge(**kwargs)
    
    @property
    def robot(**kwargs): return Robot(**kwargs)
    
    @property
    def robotfindskitten(**kwargs): return Robotfindskitten(**kwargs)
    
    @property
    def roflcopter(**kwargs): return Roflcopter(**kwargs)
    
    @property
    def rook(**kwargs): return Rook(**kwargs)
    
    @property
    def sachiko(**kwargs): return Sachiko(**kwargs)
    
    @property
    def satanic(**kwargs): return Satanic(**kwargs)
    
    @property
    def seahorse(**kwargs): return Seahorse(**kwargs)
    
    @property
    def seahorse_big(**kwargs): return Seahorse_big(**kwargs)
    
    @property
    def sheep(**kwargs): return Sheep(**kwargs)
    
    @property
    def shikato(**kwargs): return Shikato(**kwargs)
    
    @property
    def shrug(**kwargs): return Shrug(**kwargs)
    
    @property
    def skeleton(**kwargs): return Skeleton(**kwargs)
    
    @property
    def small(**kwargs): return Small(**kwargs)
    
    @property
    def smiling_octopus(**kwargs): return Smiling_octopus(**kwargs)
    
    @property
    def snoopy(**kwargs): return Snoopy(**kwargs)
    
    @property
    def snoopyhouse(**kwargs): return Snoopyhouse(**kwargs)
    
    @property
    def snoopysleep(**kwargs): return Snoopysleep(**kwargs)
    
    @property
    def spidercow(**kwargs): return Spidercow(**kwargs)
    
    @property
    def squid(**kwargs): return Squid(**kwargs)
    
    @property
    def squirrel(**kwargs): return Squirrel(**kwargs)
    
    @property
    def stegosaurus(**kwargs): return Stegosaurus(**kwargs)
    
    @property
    def stimpy(**kwargs): return Stimpy(**kwargs)
    
    @property
    def sudowoodo(**kwargs): return Sudowoodo(**kwargs)
    
    @property
    def supermilker(**kwargs): return Supermilker(**kwargs)
    
    @property
    def surgery(**kwargs): return Surgery(**kwargs)
    
    @property
    def tableflip(**kwargs): return Tableflip(**kwargs)
    
    @property
    def taxi(**kwargs): return Taxi(**kwargs)
    
    @property
    def telebears(**kwargs): return Telebears(**kwargs)
    
    @property
    def template(**kwargs): return Template(**kwargs)
    
    @property
    def threader(**kwargs): return Threader(**kwargs)
    
    @property
    def threecubes(**kwargs): return Threecubes(**kwargs)
    
    @property
    def toaster(**kwargs): return Toaster(**kwargs)
    
    @property
    def tortoise(**kwargs): return Tortoise(**kwargs)
    
    @property
    def turkey(**kwargs): return Turkey(**kwargs)
    
    @property
    def turtle(**kwargs): return Turtle(**kwargs)
    
    @property
    def tux(**kwargs): return Tux(**kwargs)
    
    @property
    def tux_big(**kwargs): return Tux_big(**kwargs)
    
    @property
    def tweety_bird(**kwargs): return Tweety_bird(**kwargs)
    
    @property
    def vader(**kwargs): return Vader(**kwargs)
    
    @property
    def vader_koala(**kwargs): return Vader_koala(**kwargs)
    
    @property
    def weeping_angel(**kwargs): return Weeping_angel(**kwargs)
    
    @property
    def whale(**kwargs): return Whale(**kwargs)
    
    @property
    def wizard(**kwargs): return Wizard(**kwargs)
    
    @property
    def wood(**kwargs): return Wood(**kwargs)
    
    @property
    def world(**kwargs): return World(**kwargs)
    
    @property
    def www(**kwargs): return Www(**kwargs)
    
    @property
    def yasuna_01(**kwargs): return Yasuna_01(**kwargs)
    
    @property
    def yasuna_02(**kwargs): return Yasuna_02(**kwargs)
    
    @property
    def yasuna_03(**kwargs): return Yasuna_03(**kwargs)
    
    @property
    def yasuna_03a(**kwargs): return Yasuna_03a(**kwargs)
    
    @property
    def yasuna_04(**kwargs): return Yasuna_04(**kwargs)
    
    @property
    def yasuna_05(**kwargs): return Yasuna_05(**kwargs)
    
    @property
    def yasuna_06(**kwargs): return Yasuna_06(**kwargs)
    
    @property
    def yasuna_07(**kwargs): return Yasuna_07(**kwargs)
    
    @property
    def yasuna_08(**kwargs): return Yasuna_08(**kwargs)
    
    @property
    def yasuna_09(**kwargs): return Yasuna_09(**kwargs)
    
    @property
    def yasuna_10(**kwargs): return Yasuna_10(**kwargs)
    
    @property
    def yasuna_11(**kwargs): return Yasuna_11(**kwargs)
    
    @property
    def yasuna_12(**kwargs): return Yasuna_12(**kwargs)
    
    @property
    def yasuna_13(**kwargs): return Yasuna_13(**kwargs)
    
    @property
    def yasuna_14(**kwargs): return Yasuna_14(**kwargs)
    
    @property
    def yasuna_16(**kwargs): return Yasuna_16(**kwargs)
    
    @property
    def yasuna_17(**kwargs): return Yasuna_17(**kwargs)
    
    @property
    def yasuna_18(**kwargs): return Yasuna_18(**kwargs)
    
    @property
    def yasuna_19(**kwargs): return Yasuna_19(**kwargs)
    
    @property
    def yasuna_20(**kwargs): return Yasuna_20(**kwargs)
    
    @property
    def ymd_udon(**kwargs): return Ymd_udon(**kwargs)
    
    @property
    def zen_noh_milk(**kwargs): return Zen_noh_milk(**kwargs)



def Cower(thoughts, eyes, eye, tongue):
  return f"""
     {thoughts}
      {thoughts}
        ,__, |    | 
        ({eyes})\\|    |___
        (__)\\|    |   )\\_
         {tongue}  |    |_w |  \\
             |    |  ||   *

             Cower....
"""


def Vader(thoughts, eyes, eye, tongue):
  return f"""
        {thoughts}    ,-^-.
         {thoughts}   !{eye}Y{eye}!
          {thoughts} /./=\\.\\______
               ##      {tongue})\\/\\
                ||-----w||
                ||      ||

               Cowth Vader
"""


def Default(thoughts, eyes, eye, tongue):
   return f"""
           {thoughts}   ^__^
            {thoughts}  ({eyes})\\_______
               (__)\\       )\\/\\
               {tongue} ||----w |
                  ||     ||
"""


from ..modules import stringWidth

SPEECH_DELIMITERS = {
  'first':  ['/', '\\'],
  'middle': ['|', '|'],
  'last':   ['\\', '/'],
  'only':   ['<', '>'],  
}

THOUGHT_DELIMITERS = {
  'first':  ['(', ')'],
  'middle': ['(', ')'],
  'last':   ['(', ')'],
  'only':   ['(', ')'],
}

def pad(text, length:int):
    listlen = length - stringWidth(text) + 1
    trailingSpaces = ' ' * listlen
    return text + trailingSpaces

def top(length):
    return f' {"_"*(length + 3)}'

def bottom(length):
    return ' ' + '_'*(length + 3)

def say(text, wrap):
  return format(text, wrap, SPEECH_DELIMITERS)

def think(text, wrap):
  return format(text, wrap, THOUGHT_DELIMITERS)

def split(text, wrap):
    #print(f"wrap: '{wrap}'")
    text = text.replace("/\r\n?|[\n\u2028\u2029]/g", '\n').replace("/^\uFEFF/", '').replace("/\t/g", '        ')
    lines = []

    if wrap:
        text = ' '.join(text.split("/\s+/"))
        start = 0
        while start < len(text):
            end = min(start + wrap, len(text))
            space = text.rfind(' ', 0, end)
            if end < len(text) and space > start:
                end = space
            lines.append(text[start:end])
            if space > start:
                start = end + 1
            else:
                start = end
                
    else:
        lines = text.split('\n')
    return lines


def format(text, wrap, delimiters):
    #print(wrap)
    lines = split(text, wrap) if wrap else [text]
    if len(lines) == 0:
        lines = ['']
    maxLength = max(map(stringWidth, lines))

    def mapfunc(line):
        i = lines.index(line)
        only = len(lines) if len(lines) == 1 else None
        first = True if i == 0 else None
        last = True if i == (len(lines)-1) else None
        _type = 'only' if only else 'first' if first else 'last' if last else 'middle'
        start, end = delimiters[_type]
        return f"{start} {pad(line, maxLength)} {end}"

    res1 = map(mapfunc, lines)
    res = list(res1)
    # print(f"list(res): '{res}'")
    balloon = [
        top(maxLength),
    ]
    for j in range(0, len(res)):
        balloon.append(res[j])
    balloon.append(bottom(maxLength))
    # print(balloon)
    return '\n'.join(balloon)


def Ghostbusters(thoughts, eyes, eye, tongue):
  return f"""
          {thoughts}
           {thoughts}
            {thoughts}          __---__
                    _-       /--______
               __--( /     \\ )XXXXXXXXXXX\\v.
             .-XXX(   {eye}   {eye}  )XXXXXXXXXXXXXXX-
            /XXX(       U     )        XXXXXXX\\
          /XXXXX(              )--_  XXXXXXXXXXX\\
         /XXXXX/ (      O     )   XXXXXX   \\XXXXX\\
         XXXXX/   /            XXXXXX   \\__ \\XXXXX
         XXXXXX__/          XXXXXX         \\__---->
 ---___  XXX__/          XXXXXX      \\__         /
   \\-  --__/   ___/\\  XXXXXX            /  ___--/=
    \\-\\    ___/    XXXXXX              '--- XXXXXX
       \\-\\/XXX\\ XXXXXX                      /XXXXX
         \\XXXXXXXXX   \\                    /XXXXX/
          \\XXXXXX      >                 _/XXXXX/
            \\XXXXX--__/              __-- XXXX/
             -XXXXXXXX---------------  XXXXXX-
                \\XXXXXXXXXXXXXXXXXXXXXXXXXX/
                  ""VXXXXXXXXXXXXXXXXXXV""
"""


def Moojira(thoughts, eyes, eye, tongue):
  return f"""
     {thoughts}              
      {thoughts}    /ss/           
   \`oys:  .dmmd\`  :syo\`   
   /dmmy   .//.   hmmd:   
    -/:\`          \`:/-    
oyo/:.     ^__^     .:/oy+
/mmmmm+   <({eyes}\)>  \`ommmmm/
 smmmmms. -(__). .smmmmmo 
 \`+dmmmmd+\`\`{tongue}\`\`+dmmmmd+  
   -ymmmmmh/\`\`+hmmmmmy-   
    \`/hmmmmmhhmmmmmh/\`    
      \`/hmmmmmmmmh/\`      
        \`/hmmmmmd/        
      \`oo.\`/dmmmmdo\`      
     \`ymmd+\`VVmmmmmy\`     
     smmmmd-  /mmmmms     
    -mmmmm+    ommmmm-    
    -ooooo\`    .ooooo.    
"""


def Ymd_udon(thoughts, eyes, eye, tongue):
  return f"""
   {thoughts}
    {thoughts}
  
             _ - ￣ - _
           _-_＿＿＿＿_- _
         ￣ｌ  ●   ●  l￣
            ヽ､_ ⌒ _ノ
         _ -‐ニ ￣ ニ‐- _
  /⌒ ‐ﾆ‐ ￣   /    \ ￣ ‐ﾆ‐⌒ヽ
 ヽ､_ノ       └-ｕ‐┘      ヽ､_ノ
"""


def Vader_koala(thoughts, eyes, eye, tongue):
  return f"""
   {thoughts}
    {thoughts}        .
     .---.  //
    Y|{eye} {eye}|Y// 
   /_(i=i)K/ 
   ~()~*~()~  
    (_)-(_)   

     Darth 
     Vader    
     koala        
"""


def Charlie(thoughts, eyes, eye, tongue):
  return f"""
  {thoughts}
   {thoughts}
    {thoughts}     ,, ＿
        ／      ｀､
       /   (_ﾉL_） ヽ
      /   ´・ ・｀  l
    （l      し     l）
      l     ＿＿    l
      >  ､ _      ィ
    ／        ￣    ヽ
   /  |              iヽ
   |＼|              |/|
   |  ||/＼／＼／＼/ | |
"""


def Dalek(thoughts, eyes, eye, tongue):
  return f"""
   {thoughts}
    {thoughts}
              ___
      D>=G==='   '.
            |======|
            |======|
        )--/]IIIIII]
           |_______|
           C O O O D
          C O  O  O D
         C  O  O  O  D
         C__O__O__O__D
snd     [_____________]
"""


def Mooghidjirah(thoughts, eyes, eye, tongue):
  return f"""
 {thoughts}       {thoughts}      {thoughts}      
  {thoughts}        ^__^  {thoughts}        
    ^__^   ({eyes})   ^__^  
    ({eyes})   (__)   ({eyes})   
    (__)    {tongue}    (__)   
oyo/:{tongue}            {tongue}:/oy+
/mmmmm+   syyyyo  \`ommmmm/
 smmmmms. -ymmy. .smmmmmo 
 \`+dmmmmd+\`\`::\`\`+dmmmmd+  
   -ymmmmmh/\`\`+hmmmmmy-   
    \`/hmmmmmhhmmmmmh/\`    
      \`/hmmmmmmmmh/\`      
        \`/hmmmmmd/        
      \`oo.\`/dmmmmdo\`      
     \`ymmd+\`\`ommmmmy\`     
     smmmmd-  /mmmmms     
    -mmmmm+    ommmmm-    
    -ooooo\`    .ooooo.     
"""


from setuptools import setup
import re

with open('cowsay/__init__.py') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('version is not set')

readme = ''
with open('README.md') as f:
    readme = f.read()

packages = [
    'cowsay',
    'cowsay.lib',
    'cowsay.lib.cows'
]

setup(
    name = "cowsay-py",
    version = version,
    packages = packages,
    license = 'MIT',
    author = "Ovlic",
    description = "Python module for cowsay",
    long_description = readme,
    long_description_content_type = "text/markdown",
    url = "https://github.com/ovlic/cowsay_py",
    project_urls = {
        "Bug Tracker": "https://github.com/ovlic/cowsay_py/issues",
    },
    classifiers = {
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    },
    include_package_data=True,
    python_requires = '>=3.6',
)


def Sachiko(thoughts, eyes, eye, tongue):
  return f"""
     {thoughts}
      {thoughts}
       {thoughts}"""+"""
             , -――- 、
          ／          ヽ、
        /爻ﾉﾘﾉﾊﾉﾘlﾉ ゝ  l
     ＜ﾉﾘﾉ‐'    ｰ  ﾘ ＞ }
        l ﾉ ┃    ┃ l ﾉ  ﾉ
        l人   r‐┐   !ﾉ＾)
           ゝ \` ´ ‐＜´
"""